#megadrive.ps1 v5.4

#Unzip ROMs That Do Not Support .ZIP Files On RetroPie
$romUnzip = $false

#Any Exact ROM Name (Including Extension) In This List Will Not Be Deleted
#-ROMs listed here will NOT be deleted even if they match a value in romsExceptions
#-ROMs listed here will NOT be deleted if it's NOT the Primary ROM via romsSortOrder
#-Ex: "Castlevania (USA) (Rev A).zip"
$romsDoNotDelete = @(
)

#Any ROMs Containing These Strings Or Exact ROM Name Will Be Removed Unless the Exact ROM Name Is In romsDoNotDelete
$romsExceptions = @(
	"[BIOS]"
	,"[b]"
    ,"(Alpha)"
	,"(Beta)"
	,"(Beta 1)"
	,"(Beta 2)"
	,"(Beta 3)"
	,"(Proto)"
	,"(Proto 1)"
	,"(Proto 2)"
	,"(Promo)"
	,"(Sample)"
	,"(Debug Version)"
	,"(Enhancement Chip)"
	,"(GameCube Edition)"
	,"(Promo, Virtual Console)"
	,"(Virtual Console)"
	,"(Wii Virtual Console)"
	,"(Wii U Virtual Console)"
	,"(Asia)"
	,"(Australia)"
	,"(Brazil)"
	,"(Canada)"
	,"(China)"
	,"(France)"
	,"(Germany)"
	,"(Hong Kong)"
	,"(Italy)"
	,"(Japan)"
	,"(Korea)"
	,"(Netherlands)"
	,"(Russia)"
	,"(Spain)"
	,"(Sweden)"
	,"(Taiwan)"
	#Demo/Test/Misc Cartridge
	,"Flux (Europe) (Requires MegaCD) (Program).zip" #Not a Game (Displays Visual Effects with Sega Mega-CD)
	,"Sega Channel (USA) (General Instrument) (Program).zip" #Not a Game (Sega Online Cartridge)
	,"Sega Channel (USA) (Scientific Atlanta) (Program).zip" #Not a Game (Sega Online Cartridge)
	,"Sega Channel Demo (USA) (Program).zip" #Not a Game (Sega Online Cartridge)
	#RAM Modification Cheat Cartridge
	,"Action Replay (Europe) (Program) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Game Genie (USA, Europe) (Program).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Game Genie (USA, Europe) (Rev A) (Program).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Pro Action Replay (Europe) (Alt 1) (Program) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Pro Action Replay (Europe) (Program) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	,"Pro Action Replay 2 (Europe) (Program) (Unl).zip" #Not a Game (RAM Modification Cheat Cartridge)
	#Duplicate ROMSs with Alternate Names
	,"Arcade Legends Sega Mega Drive ~ Arcade Legends Sega Genesis ~ Mega Drive Play TV (World).zip" #Mini console released in 2005 that includes 6 Classic Sega ROMs
	,"Arcade Legends Street Fighter II' - Special Champion Edition ~ Mega Drive Play TV 3 (World).zip" #Mini console released in 2005 that includes 6 Classic Sega ROMs
	,"Bad Omen (Japan, Korea).zip" #Alternate Name for 'Devilish - The Next Possession (USA).zip'
	,"Columns III - Taiketsu! Columns World (Japan, Korea).zip" #Alternate Name for 'Columns III (USA).zip'
	,"Flashback (Europe).zip" #Alternate Name for 'Flashback - The Quest for Identity (USA).zip'
	,"James Pond II (Japan, Korea).zip" #Alternate Name for 'James Pond II - Codename - Robocod (USA, Europe).zip'
	,"Landstalker - The Treasures of King Nole (Europe).zip" #Alternate Name for 'Landstalker (USA).zip'
	,"Mazin Wars (Europe).zip" #Alternate Name for 'Mazin Saga - Mutant Fighter (USA).zip'
	,"Mazin Saga (Japan, Korea).zip" #Alternate Name for 'Mazin Saga - Mutant Fighter (USA).zip'
	,"Mickey Mania (Europe).zip" #Alternate Name for 'Mickey Mania - The Timeless Adventures of Mickey Mouse (USA).zip'
	,"Rock 'n' Roll Racing (Europe).zip" #Alternate Name for 'Rock n' Roll Racing (USA).zip'
	,"Sonic & Knuckles (World).zip" #Alternate Name for 'Sonic & Knuckles + Sonic The Hedgehog (USA).zip'
	,"Speedball 2 (Europe).zip" #Alternate Name for 'Speedball 2 - Brutal Deluxe (USA).zip'
	,"Superman - The Man of Steel (Europe).zip" #Alternate Name for 'Superman (USA).zip'
	,"Teenage Mutant Hero Turtles - The Hyperstone Heist (Europe).zip" #Alternate Name for 'Teenage Mutant Ninja Turtles - The Hyperstone Heist (USA).zip'
	,"Teenage Mutant Hero Turtles - Tournament Fighters (Europe).zip" #Alternate Name for 'Teenage Mutant Ninja Turtles - Tournament Fighters (USA).zip'
	,"Wimbledon (Europe).zip" #Alternate Name for 'Wimbledon Championship Tennis (USA).zip'
	,"Wiz'n'Liz - The Frantic Wabbit Wescue (Europe).zip" #Alternate Name for 'Wiz'n'Liz (USA).zip'
)

#Preferred Sorting Order to Pick 1 Primary ROM to Keep When Evaluating Duplicates
#-Order Selects Specific Names followed by more generalized names based around language
#-ROMs in romsExceptions will not be deleted even if it is not determined as the primary
#-Ex: "Joust (USA).zip" would be primary over "Joust (Japan).zip"
$romsSortOrder = @(
	#---------------------------------------
	#---USA PREFERRED ORDER---
	#---------------------------------------
	#(USA).zip
	"\(USA\)\.zip"
	#(USA) (Unl).zip
	,"\(USA\)\s*\(Unl\)\.zip"
	#(USA) (En,*).zip
	,"\(USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA) (En,*) (Unl).zip
	,"\(USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Europe).zip
	,"\(USA,\s*Europe\)\.zip"
	#(USA, Europe) (Unl).zip
	,"\(USA,\s*Europe\)\s*\(Unl\)\.zip"
	#(USA, Europe) (En,*).zip
	,"\(USA,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Europe) (En,*) (Unl).zip
	,"\(USA,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, USA).zip
	,"\(Japan,\s*USA\)\.zip"
	#(Japan, USA) (Unl).zip
	,"\(Japan,\s*USA\)\s*\(Unl\)\.zip"
	#(Japan, USA) (En,*).zip
	,"\(Japan,\s*USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, USA) (En,*) (Unl).zip
	,"\(Japan,\s*USA\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, USA) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*USA\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Asia).zip
	,"\(USA,\s*Asia\)\.zip"
	#(USA, Asia) (Unl).zip
	,"\(USA,\s*Asia\)\s*\(Unl\)\.zip"
	#(USA, Asia) (En,*).zip
	,"\(USA,\s*Asia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Asia) (En,*) (Unl).zip
	,"\(USA,\s*Asia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Asia) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Asia\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"	
	#(USA, Australia).zip
	,"\(USA,\s*Australia\)\.zip"
	#(USA, Australia) (Unl).zip
	,"\(USA,\s*Australia\)\s*\(Unl\)\.zip"
	#(USA, Australia) (En,*).zip
	,"\(USA,\s*Australia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Australia) (En,*) (Unl).zip
	,"\(USA,\s*Australia\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Australia) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Australia\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA, Korea).zip
	,"\(USA,\s*Korea\)\.zip"
	#(USA, Korea) (Unl).zip
	,"\(USA,\s*Korea\)\s*\(Unl\)\.zip"
	#(USA, Korea) (En,*).zip
	,"\(USA,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(USA, Korea) (En,*) (Unl).zip
	,"\(USA,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(USA, Korea) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(USA,\s*Korea\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(USA) (Rev #)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(USA) (Rev A)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(USA) (Rev #A)
	,"^(?=.*?\(USA\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(USA) (Alt) OR (Alternate)
	,"^(?=.*?\(USA\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(USA) (Alt #)
	,"^(?=.*?\(USA\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(USA)
	,"\(USA\)"
	#(USA, Europe)
	,"\(USA,\s*Europe\)"
	#(Japan, USA)
	,"\(Japan,\s*USA\)"
	#(USA, Asia)
	,"\(USA,\s*Asia\)"
	#(USA, Australia)
	,"\(USA,\s*Australia\)"
	#(USA, Korea)
	,"\(USA,\s*Korea\)"	
	#(*USA*)
	,"\((.*?)USA(.*?)\)"
	#---------------------------------------
	#---WORLD PREFERRED ORDER---
	#---------------------------------------
	#(World).zip
	"\(World\)\.zip"
	#(World) (Unl).zip
	,"\(World\)\s*\(Unl\)\.zip"
	#(World) (En,*).zip
	,"\(World\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(World) (En,*) (Unl).zip
	,"\(World\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(World) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(World\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(World) (Rev #)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(World) (Rev A)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(World) (Rev #A)
	,"^(?=.*?\(World\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(World) (Alt) OR (Alternate)
	,"^(?=.*?\(World\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(World) (Alt #)
	,"^(?=.*?\(World\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(World)
	,"\(World\)"
	#(*World*)
	,"\((.*?)World(.*?)\)"
	#---------------------------------------
	#---EUROPE PREFERRED ORDER---
	#---------------------------------------
	#(Europe).zip
	"\(Europe\)\.zip"
	#(Europe) (Unl).zip
	,"\(Europe\)\s*\(Unl\)\.zip"
	#(Europe) (En,*).zip
	,"\(Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Europe) (En,*) (Unl).zip
	,"\(Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, Europe).zip
	,"\(Japan,\s*Europe\)\.zip"
	#(Japan, Europe) (Unl).zip
	,"\(Japan,\s*Europe\)\s*\(Unl\)\.zip"
	#(Japan, Europe) (En,*).zip
	,"\(Japan,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, Europe) (En,*) (Unl).zip
	,"\(Japan,\s*Europe\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, Europe) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*Europe\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Europe) (Rev #)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(Europe) (Rev A)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(Europe) (Rev #A)
	,"^(?=.*?\(Europe\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(Europe) (Alt) OR (Alternate)
	,"^(?=.*?\(Europe\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(Europe) (Alt #)
	,"^(?=.*?\(Europe\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(Europe)
	,"\(Europe\)"
	#(*Europe*)
	,"\((.*?)Europe(.*?)\)"
	#---------------------------------------
	#---JAPAN PREFERRED ORDER---
	#---------------------------------------
	#(Japan).zip
	"\(Japan\)\.zip"
	#(Japan) (Unl).zip
	,"\(Japan\)\s*\(Unl\)\.zip"
	#(Japan) (En,*).zip
	,"\(Japan\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan) (En,*) (Unl).zip
	,"\(Japan\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan, Korea).zip
	,"\(Japan,\s*Korea\)\.zip"
	#(Japan, Korea) (Unl).zip
	,"\(Japan,\s*Korea\)\s*\(Unl\)\.zip"
	#(Japan, Korea) (En,*).zip
	,"\(Japan,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\.zip"
	#(Japan, Korea) (En,*) (Unl).zip
	,"\(Japan,\s*Korea\)\s*\(En(,|\+|Ca|Da|De|En|Es|Fi|Fr|It|Ja|Nl|No|Pl|Pt|Sv|Zh)*?\)\s*\(Unl\)\.zip"
	#(Japan, Korea) <Anything Except (Alt ) or (Rev ) or (Demo)>.zip
	,"^(?=.*?\(Japan,\s*Korea\))((?!\(Alt\s*[A-Z0-9]{0,2}\)|\(Rev\s*[A-Z0-9\.]{0,4}\)|\(Demo\)).)*$"
	#(Japan) (Rev #)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[0-9\.]{0,4}\)).*$"
	#(Japan) (Rev A)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[A-Z]{0,4}\)).*$"
	#(Japan) (Rev #A)
	,"^(?=.*?\(Japan\))(?=.*?\(Rev\s*[A-Z0-9\.]{0,4}\)).*$"
	#(Japan) (Alt) OR (Alternate)
	,"^(?=.*?\(Japan\))(?=.*?(\(Alt\)|\(Alternate\))).*$"
	#(Japan) (Alt #)
	,"^(?=.*?\(Japan\))(?=.*?\(Alt\s*[A-Z0-9]{1,2}\)).*$"
	#(Japan)
	,"\(Japan\)"
	#(*Japan*)
	,"\((.*?)Japan(.*?)\)"
	#---------------------------------------
	#---MISC PREFERRED ORDER---
	#---------------------------------------
	#(Unknown)
	,"\(Unknown\)"
	#---------------------------------------
	#---LAST RESORT VALID ROM EXTENSION PREFERRED ORDER---
	#---------------------------------------
	,"\.zip"
	,"\.7z"
	,"\.smd"
	,"\.bin"
	,"\.gen"
	,"\.md"
	,"\.smd"
	,"\.iso"
)
